local _G = _G
local tonumber = _G.tonumber
local pairs = _G.pairs

local UnitName = _G.UnitName
local UnitAffectingCombat = _G.UnitAffectingCombat
local GetPetActionInfo = _G.GetPetActionInfo

local AceLibrary = _G.AceLibrary

local MAJOR_VERSION = "Threat-1.0"
local MINOR_VERSION = tonumber(("$Revision: 37073 $"):match("%d+"))

if _G.ThreatLib_OUT_OF_DATE or not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then
	_G.ThreatLib_OUT_OF_DATE = true
	return
end

if MINOR_VERSION > _G.ThreatLib_MINOR_VERSION then
	_G.ThreatLib_MINOR_VERSION = MINOR_VERSION
end

local ThreatLib = _G.ThreatLib

-- local _, c = _G.UnitClass("player")
-- We'll let it load for everyone, because sometimes non-hunter/locks get pets.
-- if c ~= "HUNTER" and c ~= "WARLOCK" then return end 
local c = "PET"

local BS = AceLibrary("Babble-Spell-2.2")
local Aura = AceLibrary("SpecialEvents-Aura-2.0")
ThreatLib:GetModule("ClassCore"):SetModuleDefaultState(c, false)
local Pet = ThreatLib:GetModule("ClassCore"):NewModule(c)

local growlThreatAmounts		= {50, 65, 110, 170, 240, 320, 415, 664}
local cowerThreatAmounts		= {-30, -55, -85, -125, -175, -225, -360}

local anguishThreatAmounts		= {300, 395, 632}
local cleaveThreatAmounts		= {0, 0, 0, 0, 100, 130}
local tormentThreatAmounts		= {45, 75, 125, 215, 300, 395, 632}
local sufferingThreatAmounts	= {150, 300, 450, 600, 645, 885}
local soothingKissThreatAmounts	= {-45, -75, -127, -165, -275}

local spellNames = {
	[BS["Growl"]] = true, 
	[BS["Cower"]] = true,
	[BS["Anguish"]] = true,
	[BS["Cleave"]] = true,
	[BS["Torment"]] = true,
	[BS["Suffering"]] = true,
	[BS["Soothing Kiss"]] = true
}

function Pet:ClassInit()
	-- CastHandlers
	self.petName = UnitName("pet")
	self.isPetModule = true
	self.unitType = "pet"
	
	self.CastHandlers[BS["Growl"]]			= self.Growl
	self.CastHandlers[BS["Cower"]]			= self.Cower

	self.CastHandlers[BS["Anguish"]]		= self.Anguish
	self.CastHandlers[BS["Cleave"]]			= self.Cleave
	self.CastHandlers[BS["Torment"]]		= self.Torment
	self.CastHandlers[BS["Suffering"]]		= self.Suffering
	self.CastHandlers[BS["Soothing Kiss"]]	= self.SoothingKiss
	
	self:RegisterEvent("LOCALPLAYER_PET_RENAMED")
	self:RegisterEvent("PET_ATTACK_START")
	self:RegisterEvent("UNIT_NAME_UPDATE")
	self.skillRanks = {}
	self:ScanPetSkillRanks()
	self:RegisterEvent("PET_BAR_UPDATE", "ScanPetSkillRanks")	
end

function Pet:ScanPetSkillRanks()
	for i = 1,10 do
		local name, rank = GetPetActionInfo(i)
		if spellNames[name] then
			self.skillRanks[name] = rank
		end
	end
	self:UpdateGrowlData()
end

-- Pet Growl Scaling, see http://www.wowwiki.com/Growl_%28pet%29
local petLevel
local apThreshold
local effectiveGrowlThreatAmounts = {}

function Pet:UpdateGrowlData()
	if select(2, UnitClass("player")) ~= "HUNTER" or not UnitExists("pet") or (petLevel and petLevel == UnitLevel("pet")) then
		return
	end
	petLevel = UnitLevel("pet")
	apThreshold = math.max(28.14 * petLevel - 1235.6, 0)
	local bestRank = 1 + math.floor(petLevel/10)
	if bestRank > 1 then
		for i=1,bestRank-1 do
			effectiveGrowlThreatAmounts[i] = growlThreatAmounts[i+1]
		end
	end
	-- Approximate scaling
	local baseAmount = growlThreatAmounts[bestRank]
	local amountDelta = growlThreatAmounts[math.min(bestRank+1, #growlThreatAmounts)] - baseAmount
	effectiveGrowlThreatAmounts[bestRank] = baseAmount + amountDelta *  (petLevel % 10) / 10
end

function Pet:Growl(rank)
	rank = rank or self.skillRanks[BS["Growl"]]
	local ranknum = tonumber(rank:match("%d+"))
	local threat = effectiveGrowlThreatAmounts[ranknum]
	local baseAP, posAPBuff, negAPBuff = UnitAttackPower("pet");	
	local petAP = baseAP + posAPBuff + negAPBuff;
	if petAP > apThreshold then
		threat = threat + 5.7 * (petAP - apThreshold)
	end
	self:AddTargetThreat(math.floor(threat+0.5) * self:threatMods())
end

function Pet:Cower(rank)
	rank = rank or self.skillRanks[BS["Cower"]]
	local ranknum = tonumber(rank:match("%d+"))
	self:AddTargetThreat(cowerThreatAmounts[ranknum] * self:threatMods())
end

function Pet:Anguish(rank)
	rank = rank or self.skillRanks[BS["Anguish"]]
	local ranknum = tonumber(rank:match("%d+"))
	self:AddTargetThreat(anguishThreatAmounts[ranknum] * self:threatMods())
end

function Pet:Cleave(rank)
	rank = rank or self.skillRanks[BS["Cleave"]]
	local ranknum = tonumber(rank:match("%d+"))
	self:AddTargetThreat(cleaveThreatAmounts[ranknum] * self:threatMods())
end

function Pet:Torment(rank)
	rank = rank or self.skillRanks[BS["Torment"]]
	local ranknum = tonumber(rank:match("%d+"))
	self:AddTargetThreat(tormentThreatAmounts[ranknum] * self:threatMods())
end

function Pet:Suffering(rank)
	rank = rank or self.skillRanks[BS["Suffering"]]
	local ranknum = tonumber(rank:match("%d+"))
	self:AddTargetThreat(sufferingThreatAmounts[ranknum] * self:threatMods())
end

function Pet:SoothingKiss(rank)
	rank = rank or self.skillRanks[BS["Soothing Kiss"]]
	local ranknum = tonumber(rank:match("%d+"))
	self:AddTargetThreat(soothingKissThreatAmounts[ranknum] * self:threatMods())
end

function Pet:LOCALPLAYER_PET_RENAMED()
	self.petName = UnitName("pet")
end

function Pet:UNIT_NAME_UPDATE(arg1)
	if arg1 == "pet" then	
		self.petName = UnitName("pet")
		if self.petName then
			ThreatLib:PARTY_MEMBERS_CHANGED()
			self:UpdateGrowlData()
		end		
	end
end

-- Poll every half second to see if the pet is out of combat, since there isn't a good pet-out-of-combat event.
local function checkPetOutOfCombat(this, elapsed)
	local self = this.self
	local t = this.timeSinceLastCheck + elapsed
	if t > 0.5 then
		this.timeSinceLastCheck = 0
		if not UnitAffectingCombat("pet") then
			this:SetScript("OnUpdate", nil)
			self:PET_REGEN_ENABLED()
		end
	else
		this.timeSinceLastCheck = t
	end
end

-- The pet shouldn't be able to generate any threat without attacking, so we should be ok with this event.
function Pet:PET_ATTACK_START()
	self:PET_REGEN_DISABLED()
	local f = self.inCombatFrame
	if not f then
		f = _G.CreateFrame("Frame", "ThreatLibPetInCombatFrame", _G.UIParent)
		self.inCombatFrame = f
		f.self = self
	end
	f:SetScript("OnUpdate", checkPetOutOfCombat)
	f.timeSinceLastCheck = 0
end

_G.tinsert(ThreatLib.UpvalueFixers, function(lib) ThreatLib = lib end)
